# This file is part of a program "django-dialogform", a django app to load and open form views
# within <dialog> html element popups.

# Copyright (C) 2023, Zoltan Kemenczy

# This program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome
# to redistribute it under conditions of the GPLv3 LICENSE included in this package
# To use it, refer to the included README.rst

from django.views.generic.edit  import FormMixin, FormView
from django.http 		import JsonResponse
from django.core.exceptions	import ImproperlyConfigured

from .forms import *

class DialogFormMixin(FormMixin):
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["is_dialog"] = True
        return context
    
    def get_template_names(self):
        try:
            return super().get_template_names()
        except ImproperlyConfigured:
            # Default for simple DialogViews:
            return ["dialogform/dialog.html"]

    def form_valid(self, form):
        response = super().form_valid(form)
        # response should be a url with status 302 (redirect)
        json_response = JsonResponse({'url':    response.url,
                                      'status': response.status_code})
        return json_response

class DialogFormView(DialogFormMixin, FormView):
    pass



# This file is part of a program "django-dialogform", a django app to load and open form views
# within <dialog> html element popups.

# Copyright (C) 2023, Zoltan Kemenczy

# This program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome
# to redistribute it under conditions of the GPLv3 LICENSE included in this package
# To use it, refer to the included README.rst

"""
ASGI config

It exposes the ASGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/dev/howto/deployment/asgi/
"""

import os

from django.core.asgi import get_asgi_application

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'devtest.settings')

application = get_asgi_application()



# This file is part of a program "django-dialogform", a django app to load and open form views
# within <dialog> html element popups.

# Copyright (C) 2023, Zoltan Kemenczy

# This program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome
# to redistribute it under conditions of the GPLv3 LICENSE included in this package
# To use it, refer to the included README.rst

from django.db import models
from django import forms
from django.forms.utils import RenderableFormMixin

class DialogMixin(RenderableFormMixin, metaclass = forms.MediaDefiningClass):
    method = "post"
    render_as = "div"
    def as_render_as(self):
        if self.render_as.lower() not in ('div','table','p','ul'):
            raise Exception(f"{self.__class__}: unrecognized form render_as attribute: {self.render_as}")
        return eval(f"self.as_{self.render_as.lower()}()")
    
class DialogForm(DialogMixin, forms.Form):
    pass




# This file is part of a program "django-dialogform", a django app to load and open form views
# within <dialog> html element popups.

# Copyright (C) 2023, Zoltan Kemenczy

# This program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome
# to redistribute it under conditions of the GPLv3 LICENSE included in this package
# To use it, refer to the included README.rst

from django.db import models
from django import forms
from django.forms.widgets import SplitDateTimeWidget
from django.contrib import admin
from django.contrib.admin.widgets import AdminSplitDateTime, AutocompleteSelectMultiple,\
    RelatedFieldWidgetWrapper

from dialogform.forms import DialogMixin
from .models import Note, Tag

class NoteForm(DialogMixin, forms.ModelForm):
    class Meta:
        model = Note
        fields = ('content', 'date', 'published', 'parents')
        field_classes = {
            'date' : forms.SplitDateTimeField
        }
        widgets = {
            'date': SplitDateTimeWidget(
                date_format = '%Y-%m-%d',
                time_format = '%H:%M',
                date_attrs = {'type':'date',
                              'class':'datetime-date'},
                time_attrs = {'type':'time',
                              'class':'datetime-time',
                              'min':'00:00',
                              'max':'23:59'})
        }
    class Media:
        js = ['dialogform/demo/js/additional.js']
        css = {'all': ['dialogform/demo/css/style.css',
                       'dialogform/demo/css/additional.css']}

    def clean_content(self):
        value = self.cleaned_data['content']
        unwanted_str = "!@#$%^&*()|\\~`'\""
        unwanted_set = set(unwanted_str)
        if not set(value).isdisjoint(unwanted_set):
            raise forms.ValidationError(
                f'content field should not contain any of: "{unwanted_str}".')
        return value

# A reverse ManyToMany relation editing (dialog) form
class NoteTagsSelectForm(DialogMixin, forms.ModelForm):
    tags = forms.ModelMultipleChoiceField(queryset=Tag.objects.all(),
                                          required=False, widget=forms.CheckboxSelectMultiple)
    class Meta:
        model = Note
        fields = ('tags',)

    # tags are not defined on Note but on Tag. So override the m2m save method
    # called from ModelForm.save() to save submitted tags relations to note (the
    # 'public' m2m_save() method is not called from ModelForm.save() in Django
    # 4.1)
    def _save_m2m(self):
        self.instance.tags.set(self.cleaned_data['tags'])


class SearchForm(DialogMixin, forms.Form):
    method = "GET"
    search = forms.CharField(required=False, initial="", widget=forms.TextInput(attrs={'size':15}))
    class Media:
        js=('dialogform/demo/js/search.js',)


# There's no django 'AdminForm' to provide the media necessary for admin except
# through ModelAdmin, so here's a utility mixin to be used for dialog forms with
# Admin widgets.
class MetaModel(models.Model): pass

class DialogAdminFormMixin(DialogMixin):
    class Media:
        js = admin.ModelAdmin(MetaModel,admin.sites.site).media._js
        # Todo: merge css dict from ModelAdmin if/when it becomes non-empty


class Note4AdminForm(DialogAdminFormMixin, NoteForm):
    parents = forms.ModelMultipleChoiceField(
        required=False, queryset=Note.objects.all(),
        widget = RelatedFieldWidgetWrapper(
            AutocompleteSelectMultiple(
                Note.parents.field,
                admin.sites.site
            ),
            Note.parents.rel,
            admin.sites.site,
            can_add_related=True)
    )
    class Media:
        js = ['dialogform/demo/js/admin_cleanup.js']  # to clean up admin
                                                      # datetime shortcuts from
                                                      # document after closing
        
    class Meta(NoteForm.Meta):
        model = Note
        fields = ('content', 'date', 'parents', 'published')
        widgets = {
            'date': AdminSplitDateTime(
                attrs={'format':['%Y-%m-%d','%H:%M']})
        }



# This file is part of a program "django-dialogform", a django app to load and open form views
# within <dialog> html element popups.

# Copyright (C) 2023, Zoltan Kemenczy

# This program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome
# to redistribute it under conditions of the GPLv3 LICENSE included in this package
# To use it, refer to the included README.rst

"""
Django settings for dialogform/demo project.

Generated by 'django-admin startproject' using Django 4.2.dev20221221104129.

For more information on this file, see
https://docs.djangoproject.com/en/dev/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/dev/ref/settings/
"""

from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent.parent


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/dev/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'django-insecure-!#qk$%u_18(+9q1(kpowrb@shh-y897epugm027y7lnt682ga@'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'dialogform',
    'dialogform.demo',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'dialogform.demo.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [ BASE_DIR ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'dialogform.demo.wsgi.application'


# Database
# https://docs.djangoproject.com/en/dev/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'dialogform/demo/db.sqlite3',
    }
}

# Password validation
# https://docs.djangoproject.com/en/dev/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/dev/topics/i18n/

LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'America/Toronto'
USE_I18N = True
USE_L10N = True
USE_TZ = True
TIME_INPUT_FORMATS = ['%H:%M']


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/dev/howto/static-files/

STATIC_URL = 'static/'

# Default primary key field type
# https://docs.djangoproject.com/en/dev/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

X_FRAME_OPTIONS = 'SAMEORIGIN'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'default': {
            'format': '{name} {levelname}: {message}',
            'style': '{',
        },
        'verbose': {
            'format': '{name} {asctime} {module} {process:d} {thread:d} {levelname}: {message}',
            'style': '{',
        },
    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'default',
        },
    },
    'root': {
        'handlers': ['console'],
        'level': 'INFO',
    },
    'loggers': {
    },
}

LOGGING['loggers']['django.request'] = {
    'handlers': ['console'],
    'level': 'INFO',
    'propagate': False,
}



# This file is part of a program "django-dialogform", a django app to load and open form views
# within <dialog> html element popups.

# Copyright (C) 2023, Zoltan Kemenczy

# This program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome
# to redistribute it under conditions of the GPLv3 LICENSE included in this package
# To use it, refer to the included README.rst

from django.test import TestCase

# See demo/tests.py. If installed, run by ./manage.py test



# This file is part of a program "django-dialogform", a django app to load and open form views
# within <dialog> html element popups.

# Copyright (C) 2023, Zoltan Kemenczy

# This program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome
# to redistribute it under conditions of the GPLv3 LICENSE included in this package
# To use it, refer to the included README.rst

from django.shortcuts import render
from django.views.generic import ListView
from django.views.generic.edit import FormMixin, UpdateView
from django.urls import reverse, reverse_lazy


from dialogform.views import DialogFormMixin
from .models import Note, Tag
from .forms import NoteForm, NoteTagsSelectForm, SearchForm, Note4AdminForm

class Notes(ListView):
    model = Note
    template_name = "dialogform/demo/note_list.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        queryform = \
            queryform = SearchForm(self.request.GET) if 'search' in self.request.GET \
                else SearchForm({'search':''})
        context["queryform"] = queryform
        queryform.full_clean()
        qd = queryform.cleaned_data
        if qd["search"]:
            context['note_list'] = context['note_list'].filter(content__icontains=qd["search"])
        return context

class NoteChange(DialogFormMixin, UpdateView):
    model = Note
    template_name = "dialogform/demo/note_form.html"
    extends = "dialogform/dialog.html"
    
    def get_form_class(self):
        return NoteForm if 'admin' not in self.kwargs else Note4AdminForm

    def get_success_url(self):
        return reverse("notes") if 'admin' not in self.kwargs \
            else reverse("admin:demo_note_changelist")
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if 'admin' in self.kwargs:
            context["has_admin"] = True
        context["dialogform_template"] = self.extends
        return context

class NoteChangeIframe(NoteChange):
    extends = "dialogform/page.html"


class NoteSelectTags(DialogFormMixin, UpdateView):
    model = Note
    form_class = NoteTagsSelectForm
    template_name = "dialogform/dialog.html"

    def get_success_url(self):
        return reverse("notes") if 'admin' not in self.kwargs \
            else reverse("admin:demo_note_changelist")

    def get_initial(self):
        return {'tags':list(self.object.tags.values_list('id',flat=True))}




# This file is part of a program "django-dialogform", a django app to load and open form views
# within <dialog> html element popups.

# Copyright (C) 2023, Zoltan Kemenczy

# This program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome
# to redistribute it under conditions of the GPLv3 LICENSE included in this package
# To use it, refer to the included README.rst

from django.db import models
from django.contrib import admin
from django.utils.html import format_html

class Note(models.Model):
    content = models.CharField(max_length=200) 
    date = models.DateTimeField('date written')
    published = models.BooleanField(default=False)
    parents = models.ManyToManyField('self', blank=True, symmetrical=False,
                                     related_name='children')
    def __str__(self): return f'Note({self.pk}): {self.content}'


class Tag(models.Model):
    name = models.CharField(max_length=32, unique=True)
    notes = models.ManyToManyField('Note', blank=True, related_name='tags')

    def __str__(self): return self.name



# This file is part of a program "django-dialogform", a django app to load and open form views
# within <dialog> html element popups.

# Copyright (C) 2023, Zoltan Kemenczy

# This program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome
# to redistribute it under conditions of the GPLv3 LICENSE included in this package
# To use it, refer to the included README.rst

from django.db import models
from django    import forms
from django.contrib     import admin
from django.http        import JsonResponse
from django.utils.html  import format_html
from django.urls        import reverse_lazy

from .models import Note, Tag
from .forms  import *

@admin.register(Note)
class NoteAdmin(admin.ModelAdmin):
    list_display = ('noteandtags', 'dialogedit', 'date', 'published')
    list_editable = ('published',)
    
    class Media:
        js = ["dialogform/js/dialogform.js"]
        css = {"all": [
            "dialogform/css/dialogform.css",
            "dialogform/demo/css/style.css"
        ]}
    fields = ('content', 'date', 'parents', 'published')
    search_fields = ('content',)
    autocomplete_fields = ('parents',)
    
    add_form_template = "admin/change_form.html"
    change_form_template = "dialogform/demo/admin_note_change.html"
    
    def change_view(self, request, object_id, form_url='', extra_context={}):
        extra_context.update({'is_popup': True})
        response = super().change_view(request, object_id, form_url, extra_context)
        if request.method == 'POST' and response.status_code == 302:
            response = JsonResponse(
                {'url':response.url, 'status': response.status_code})
        # else return response as is
        return response

    # Provides a dialog/form popup
    @admin.display(description='Note (dialog)')
    def dialogedit(self, note):
        url = reverse_lazy("note-edit-admin", args=(note.pk,))
        return format_html(
            '<div class="dialog-anchor"' +
            '  data-type="dialog"' + # also default if omitted
            f' data-url="{url}" title="{url}"' +
            f' data-cleanup="admin_cleanup"><span>{note.content}</span></div>')

    # Admin function that display the note content field annotated with any tags
    # and a popup tags dialog. A concatenation of iframe note and tags edit dialogs
    @admin.display(description="Note/Tags (iframe-dialog)")
    def noteandtags(self, note):
        tags = note.tags.all()
        tags_html = ','.join([f'{tag.name}' for tag in tags])
        tags_html = f'<span>{tags_html}</span>' if tags \
            else '<img class="hide" src="/static/dialogform/demo/img/icon-tag.svg">'
        url1 = reverse_lazy("admin:demo_note_change", args=(note.pk,))
        url2 = reverse_lazy("note-selecttags-admin",  args=(note.pk,))
        return format_html(
            '<div class="dialog-anchor noteandtags"' +
            '  data-type="iframe"' +
            f' data-url="{url1}" title={url1}"><span>{note.content}</span></div>' + 
            f'   <sup><div class="dialog-anchor" data-url="{url2}" title="{url2}">{tags_html}' +
            '    </div></sup>')
 
@admin.register(Tag)
class TagAdmin(admin.ModelAdmin):
    pass



# This file is part of a program "django-dialogform", a django app to load and open form views
# within <dialog> html element popups.

# Copyright (C) 2023, Zoltan Kemenczy

# This program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome
# to redistribute it under conditions of the GPLv3 LICENSE included in this package
# To use it, refer to the included README.rst

from pathlib import Path
from django.contrib import admin
from django.urls import include, path, re_path
from django.views import static
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.views.i18n import JavaScriptCatalog

from .settings import BASE_DIR
from .views import Notes, NoteChange, NoteChangeIframe, NoteSelectTags

urlpatterns = [
    path('',                          Notes.as_view(),            name = 'notes'),
    path('note/<int:pk>/change/',     NoteChange.as_view(),       name = 'note-edit'),
    path('note/<int:pk>/iframe/',     NoteChangeIframe.as_view(), name = 'note-iframe'),
    path('note/<int:pk>/selecttags/', NoteSelectTags.as_view(),   name = 'note-selecttags'),

    # Urls for dialog-anchors in demo.admin.ModelAdmin notes listing
    path('note/<int:pk>/change-admin/',    NoteChange.as_view(),     {'admin':True},
                                                                  name = 'note-edit-admin'),
    path('note/<int:pk>/iframe-admin/',    NoteChangeIframe.as_view(),{'admin':True},
                                                                  name = 'note-iframe-admin'),
    path('note/<int:pk>/selecttags-admin/',NoteSelectTags.as_view(), {'admin':True},
                                                                  name = 'note-selecttags-admin'),

    re_path('.*admin/jsi18n/', JavaScriptCatalog.as_view(),            name = 'javascript-catalog'),
    path('admin/', admin.site.urls),
    re_path('^(?P<path>favicon.ico)$', static.serve, {'document_root': Path(BASE_DIR,'dialogform/static/dialogform/demo/img')}),

] + staticfiles_urlpatterns()




# This file is part of a program "django-dialogform", a django app to load and open form views
# within <dialog> html element popups.

# Copyright (C) 2023, Zoltan Kemenczy

# This program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome
# to redistribute it under conditions of the GPLv3 LICENSE included in this package
# To use it, refer to the included README.rst

from django.test import TestCase, Client
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from django.utils import timezone
from django.db.models import QuerySet
from django.contrib.auth.models import User
from dialogform.demo.models import *
from django.conf import settings

from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service as ChromeService
from selenium.webdriver.firefox.service import Service as FirefoxService
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions
from webdriver_manager.chrome import ChromeDriverManager
from webdriver_manager.firefox import GeckoDriverManager

def setUpDatabase(obj):
    # This is set up before each of the tests below
    note1 = Note.objects.create(pk=1, content="First Note", date = timezone.now(),
                                published=False)
    note1.parents.set([])
        
    note2 = Note.objects.create(pk=2, content="Second Note", date = timezone.now(),
                                published=False)
    note2.parents.set([note1])
    
    note3 = Note.objects.create(pk=3, content="Third Note", date = timezone.now(),
                                published=False)
    note3.parents.set([note1,note2])
    
    tag1 = Tag.objects.create(pk=1, name="one")
    tag1.notes.set([note1, note2])
    
    tag2 = Tag.objects.create(pk=2, name="two")
    tag2.notes.set([note2])

        
class Basic(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.driver = Client()
        
    @classmethod
    def setUpTestData(cls):
        setUpDatabase(cls)
        
    def test_note_and_tags_load(self):
        """Test notes and tags load correctly"""
        note1 = Note.objects.get(pk=1)
        note2 = Note.objects.get(pk=2)
        note3 = Note.objects.get(pk=3)
        self.assertEqual(set(note1.parents.all()), set())
        self.assertEqual(set(note2.parents.all()), set([note1]))
        self.assertEqual(set(note3.parents.all()), set([note1, note2]))
        tag1 = Tag.objects.get(pk=1)
        tag2 = Tag.objects.get(pk=2)
        self.assertEqual(set(note1.tags.all()), set([tag1]))
        self.assertEqual(set(note2.tags.all()), set([tag1,tag2]))

    def test_note_list(self):
        response = self.driver.get('/')
        self.assertEqual(response.context['note_list'].count(), 3)


class LiveServerTests:
    def setUp(self):
        setUpDatabase(self)
    
    def test_note_list(self):
        """Test note list loads"""
        self.driver.get(f'{self.live_server_url}/')
        anchors = self.driver.find_elements(By.CSS_SELECTOR, '.dialog-anchor')
        self.assertEqual(len(anchors), 4*3+3+1) # Notes, Tags, Search

    def test_note_anchor(self):
        '''dialog with django.forms.widgets'''
        self.driver.get(f'{self.live_server_url}/')
        self.driver.find_element(
            By.CSS_SELECTOR, '.dialog-anchor[data-url="/note/1/change/"]').click()
        self.driver.find_element(
            By.CSS_SELECTOR, 'form[method="dialog"] button[value="confirm"]').click()

    def test_note_anchor_iframe(self):
        '''dialog/iframe with django.forms.widgets'''
        self.driver.get(f'{self.live_server_url}/')
        self.driver.find_element(
            By.CSS_SELECTOR, '.dialog-anchor[data-url="/note/1/iframe/"]').click()
        iframe = self.driver.find_element(By.CSS_SELECTOR, 'dialog > iframe')
        self.driver.switch_to.frame(iframe)
        self.driver.find_element(
            By.CSS_SELECTOR, 'form[method="dialog"] button[value="confirm"]').click()

    def test_note_anchor_admin(self):
        '''dialog with admin widgets'''
        self.driver.get(f'{self.live_server_url}/')
        self.driver.find_element(
            By.CSS_SELECTOR, '.dialog-anchor[data-url="/note/1/change-admin/"]').click()
        self.driver.find_element(
            By.CSS_SELECTOR, 'form[method="dialog"] button[value="confirm"]').click()

    def test_note_anchor_iframe_admin(self):
        '''dialog/iframe with Admin widgets'''
        self.driver.get(f'{self.live_server_url}/')
        self.driver.find_element(
            By.CSS_SELECTOR, '.dialog-anchor[data-url="/note/1/iframe-admin/"]').click()
        iframe = self.driver.find_element(By.CSS_SELECTOR, 'dialog > iframe')
        self.driver.switch_to.frame(iframe)
        self.driver.find_element(
            By.CSS_SELECTOR, 'form[method="dialog"] button[value="confirm"]').click()

    def test_note_search_anchor(self):
        '''Local "search" dialog '''
        self.driver.get(f'{self.live_server_url}/')
        self.driver.find_element(
            By.CSS_SELECTOR, '.dialog-anchor[data-url="#local_note_dialog"]').click()
        self.driver.find_element(
            By.CSS_SELECTOR, 'form[method="dialog"] button[value="confirm"]').click()


    def admin_login(self):
        User.objects.create_superuser(username="admin", email=None, password="admin")
        self.driver.get(f'{self.live_server_url}/admin/demo/note/')
        username = self.driver.find_element(By.NAME, "username")
        if username:
            username.send_keys('admin')
            self.driver.find_element(By.NAME, "password").send_keys('admin')
            self.driver.find_element(
                By.CSS_SELECTOR, 'input[type="submit"]').click()
        self.driver.find_element(By.CSS_SELECTOR, 'th[class="column-noteandtags"]')
        
    def test_note_admin_change_form(self):
        '''Test admin view dialog/iframe'''
        self.admin_login()
        self.driver.find_element(
            By.CSS_SELECTOR, '.dialog-anchor[data-url="/admin/demo/note/1/change/"]').click()
        iframe = self.driver.find_element(By.CSS_SELECTOR, 'dialog > iframe')
        self.driver.switch_to.frame(iframe)
        self.driver.find_element(
            By.CSS_SELECTOR,'form[method="dialog"] button[value="confirm"]').click()

    def test_note_admin_non_view(self):
        ''' Test non-admin view dialog within admin'''
        self.admin_login()
        self.driver.find_element(
            By.CSS_SELECTOR, '.dialog-anchor[data-url="/note/1/change-admin/"]').click()
        self.driver.find_element(
            By.CSS_SELECTOR,'form[method="dialog"] button[value="confirm"]').click()


class Chrome(LiveServerTests, StaticLiveServerTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.driver = webdriver.Chrome(service=ChromeService(ChromeDriverManager().install()))
        cls.driver.implicitly_wait(10)

    @classmethod
    def tearDownClass(cls):
        cls.driver.quit()     # Leave during test development
        super().tearDownClass()

class Firefox(LiveServerTests, StaticLiveServerTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.driver = webdriver.Firefox(service=FirefoxService(GeckoDriverManager().install()))
        cls.driver.implicitly_wait(10)

    @classmethod
    def tearDownClass(cls):
        cls.driver.quit()     # Leave during test development
        super().tearDownClass()
